from sklearn.exceptions import NotFittedError
from pandas import DataFrame
from sklearn.base import TransformerMixin
import numpy as np
from scipy import sparse

class Vectorizer(TransformerMixin):

    def __init__(self, weights=None):
        self.value_dict = None
        self.weights = weights

    @property
    def names(self):
        if self.value_dict is None:
            raise NotFittedError("{} has not been fitted".format(self.__class__.__name__))
        self._names = [
            str(self._cols[i]) + '=' + str(value)
            for i, (name, uniques) in enumerate(self.value_dict.items())
            for value in uniques
        ]
        return self._names

    def fit(self, X, y=None):
        n, m = X.shape
        self._cols = list(X.columns) if isinstance(X, DataFrame) else list(range(m))
        X = np.array(X)
        self.value_dict = {f : np.unique(X[:, f]) for f in range(m)}
        w = self.weights if self.weights is not None else {}
        self.weights = {f: w.get(name, 1.0) for f, name in enumerate(self._cols)}
        return self

    def transform(self, X, y=None):
        mat = np.array(X)
        vals = []
        for feature, uniques in self.value_dict.items():
            seen = np.in1d(mat[:, feature], uniques)
            indices = np.searchsorted(uniques, mat[seen, feature])
            data = np.ones((indices.shape[0], )) * self.weights.get(feature)
            indptr = np.hstack([0, np.cumsum(seen)])
            vals.append(
                sparse.csr_matrix((data, indices, indptr),
                                  shape=(X.shape[0], len(uniques)))
            )
        arr = sparse.hstack(vals).toarray()
        if isinstance(X, DataFrame):
            return DataFrame(arr, columns=self.names, index=X.index)
        return arr

