# -*- coding: utf-8 -*-
"""Filter stop words from text strings


"""
from sklearn.base import TransformerMixin, BaseEstimator
from string import punctuation

class FilterStopWords(BaseEstimator, TransformerMixin):
    """
    Filter stop words from text strings.


    Parameters
    ----------
    language : str, default ('english')
        currently 'english' and 'spanish' have been tested.

    Examples
    --------
    >>> import nltk
    >>> import pandas as pd
    >>> from FilterStopWords import FilterStopWords
    >>> df = pd.DataFrame(nltk.corpus.gutenberg \
    ...                  .open('shakespeare-macbeth.txt').readlines(),columns=['text'])
    >>> filt_sw=FilterStopWords(text_cols=['text'])
    >>> filt_sw.transform(df)

    """
    def __init__(self, text_cols=None,
                 language="english",
                 make_lower=True,
                 remove_punctuation=True):
        self.text_cols = text_cols
        self.language = language
        self.make_lower = make_lower
        self.remove_punctuation = remove_punctuation
        from nltk.corpus import stopwords
        self._stop_words = stopwords.words(language)

    def fit(self, X, y=None):
        return self

    def transform(self, X, y=None):
        X = X.copy()
        for column in self.text_cols:
            text = X[column]
            if self.make_lower:
                text = text.str.lower()
            if self.remove_punctuation:
                text = text.str.translate(
                    str.maketrans({key: None
                    for key in punctuation})
                    )
            X[column + '_filt_stopwords'] = text.apply(
                lambda x: ' '.join([item for item in x.split()
                    if item not in self._stop_words]))
        return X
