from sklearn.base import TransformerMixin, BaseEstimator
import pandas as pd
import numpy as np


class OneEachEstimator(TransformerMixin, BaseEstimator):
    """
    Given the fields in fit_fields, this estimator will group the training set by
    these fields, and initialize one model per unique combination of fit_fields.

    It then proceeds to train these models on their respective subsets of the train
    set. This estimator also implements predict, where each observation in the test
    set will be 'piped' to the relevant estimator and predicted there.

    """

    def __init__(self, fit_fields=(), model=None, pipe=None, **kwargs):
        """

        :param fit_fields: fields which determine the grouping of observations
        :param model: the BaseEstimator which will be initialized for each group
        :param pipe: a sklearn.pipeline.Pipeline to fit_transform the data.
        :param kwargs: Optional arguments for the models.
        """
        self.fit_fields = fit_fields
        self.model = model
        self.pipe = pipe
        self.model_kwargs = kwargs
        self.models = None

    def fit(self, X, y=None):
        """
        For each unique combination found in X[self.fit_fields] we initialize self.model
        and continue to train on the subset of data which matches that unique combination.

        """
        uniques = X[self.fit_fields].drop_duplicates() if len(self.fit_fields) else X.drop_duplicates()
        self.models = {tuple(row): self.model(**self.model_kwargs) for index, row in uniques.iterrows()}
        transformed = pd.DataFrame(self.pipe.fit_transform(X, y), index=X.index)
        for key, model in self.models.items():
            subset = X.loc[(X[self.fit_fields] == pd.Series(key, index=self.fit_fields)).all(axis=1), :]
            idx = subset.index
            model.fit(transformed.ix[idx], y.ix[idx])
        return self

    def predict(self, X):
        """
        Pipe the correct rows to the correct model in self.models, transform the data and
        predict using the correct model before concatenating in the end.

        """
        predictions = []
        uniques = X[self.fit_fields].drop_duplicates() if len(self.fit_fields) else X.drop_duplicates()
        transformed = pd.DataFrame(self.pipe.transform(X), index=X.index)
        for index, row in uniques.iterrows():
            subset = transformed.ix[X.loc[(X[self.fit_fields] == row).all(axis=1), :].index]
            predictions.extend(self.models[tuple(row[self.fit_fields])].predict(subset))
        return np.array(predictions)